/* -------------------------------------------------------
                        simon.c

     MCU   :  CH32V003
     Takt  :

     08.05.2025  R. Seelig
   ------------------------------------------------------ */

#include <stdint.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "ch32fun.h"
#include "ch32v003_gpio.h"

#include "my_printf.h"
#include "st77xx_display_v2.h"
#include "gfx_pictures.h"

// Bilddateien
#include "led_aus.h"
#include "led_gruen.h"
#include "led_rot.h"
#include "led_blau.h"
#include "led_gelb.h"
#include "led_palette.h"

// Koordinaten der Anzeigeposition fuer die virtuellen LED's
#define gruen_x       50
#define gruen_y       16
#define blau_x        20
#define blau_y        46
#define gelb_x        80
#define gelb_y        46
#define rot_x         50
#define rot_y         76


  #define buttonleft_init()    PD2_input_init()
  #define is_buttonleft()      (!(is_PD2()))
  #define buttonright_init()   PA1_input_init()
  #define is_buttonright()     (!(is_PA1()))
  #define buttonup_init()      PA2_input_init()
  #define is_buttonup()        (!(is_PA2()))
  #define buttondwn_init()     PC0_input_init()
  #define is_buttondwn()       (!(is_PC0()))

  #define button_pullup()      { PA1_set(); PA2_set(); PD2_set(); PC0_set(); }
  #define button_init()        { button_pullup(); buttonleft_init(); buttonright_init(); buttondwn_init(); buttonup_init(); }

  #define entpr_time         40           // Tastenentprellzeit in mS
  #define double_klicktime  100           // Zeit, die ein Doppelklick auf die Taste UP verstreichen                                          // darf, um sie als Doppelklick zu erkennen

  enum { NO_KEY, UP, LEFT, RIGHT, DOWN, ENTER };


#define printf     my_printf            // "System-printf" durch eigenes ersetzen

/* -------------------------------------------------------
                          colvalue
                    nimmt RGB-Farben auf
   ------------------------------------------------------- */
struct colvalue
{
  uint8_t r;
  uint8_t g;
  uint8_t b;
};

// liefert einen 16-Bit RGB565 Farbwert aus einer Farbpalette,
// hier die Palette aus icon04
#define rgbfrompal(nr)   ( (led_pal[nr]) )

/* --------------------------------------------------------
   my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}

enum { ledgruen, ledrot, ledblau, ledgelb };

void ledctrl(char lednr, char set)
{
  switch (lednr)
  {
    // gruen
    case 0 :
    {
      if (set== 1)
      {
        bmp256_show(gruen_x,gruen_y,&ledgruen_bmp[0], &led_pal[0]);
      }
      else
      {
        bmp256_show(gruen_x,gruen_y,&ledaus_bmp[0], &led_pal[0]);
      }
      break;
    }

    // rot
    case 1 :
    {
      if (set== 1)
      {
        bmp256_show(rot_x,rot_y,&ledrot_bmp[0], &led_pal[0]);
      }
      else
      {
        bmp256_show(rot_x,rot_y,&ledaus_bmp[0], &led_pal[0]);
      }
      break;
    }

    // blau
    case 2 :
    {
      if (set== 1)
      {
        bmp256_show(blau_x,blau_y,&ledblau_bmp[0], &led_pal[0]);
      }
      else
      {
        bmp256_show(blau_x,blau_y,&ledaus_bmp[0], &led_pal[0]);
      }
      break;
    }

    // gelb
    case 3 :
    {
      if (set== 1)
      {
        bmp256_show(gelb_x,gelb_y,&ledgelb_bmp[0], &led_pal[0]);
      }
      else
      {
        bmp256_show(gelb_x,gelb_y,&ledaus_bmp[0], &led_pal[0]);
      }
      break;
    }
  }
}

/* -------------------------------------------------------
                        button_get

     wartet auf einen Tastendruck und gibt den Wert der
     Taste zurueck.

     Rueckgabe: Tastenwert  Funktion
                    1 ------ Taste oben
                    2 ------ Taste links
                    3 ------ Taste rechts
                    4 ------ Taste unten
                    5 ------ Doppelt gedrueckte Taste
                             oben (Enter)
   ------------------------------------------------------- */
uint8_t button_get(void)
{
  uint8_t  result = 0;
  uint32_t tcount;
  uint8_t  doubleklick;

  while (!result)
  {
    if (is_buttonup())
    {
      delay(entpr_time);
      while(is_buttonup());
      delay(entpr_time);

      // erfassen, ob ein doppeltes Betaetigen des Tasters erfolgt ist
      tcount= 0;
      doubleklick= 0;
      while ((tcount< double_klicktime) && (!doubleklick))
      {
        if (is_buttonup()) doubleklick= 1;
        tcount++;
        delay(1);
      }

      if (doubleklick)
      {
        delay(entpr_time);
        while(is_buttonup());
        delay(entpr_time);
        result= 5;
      }
      else
      {
        result= 1;
      }
    }
    if (is_buttonleft())
    {
      delay(entpr_time);
      while(is_buttonleft());
      delay(entpr_time);
      result= 2;
    }
    if (is_buttonright())
    {
      delay(entpr_time);
      while(is_buttonright());
      delay(entpr_time);
      result= 3;
    }
    if (is_buttondwn())
    {
      delay(entpr_time);
      while(is_buttondwn());
      delay(entpr_time);
      result= 4;
    }
  }
  return result;
}

/* --------------------------------------------------------
                            main
   -------------------------------------------------------- */
int main(void)
{
  char i;

  SystemInit();

  lcd_init();
  button_init();

  lcd_orientation(0);
  printfkomma= 2;

  bkcolor= rgbfromvalue(0,0,70);
  clrscr();
  fillrect(0,128,127,159, 0);
  fillcircle(64,60,30, rgbfromvalue(214,214,214));

  textcolor= rgbfromega(14);
  gotoxy(2,14); printf("Keyboard-Test");

  ledctrl(ledgruen, 0);
  ledctrl(ledblau, 0);
  ledctrl(ledrot, 0);
  ledctrl(ledgelb, 0);

  while(1)
  {
    i= button_get();
    switch (i)
    {
      // oben
      case 1 : ledctrl(ledgruen,1); break;
      // links
      case 2 : ledctrl(ledblau,1); break;
      // rechts
      case 3 : ledctrl(ledgelb,1); break;
      // unten
      case 4 : ledctrl(ledrot,1); break;
      // Doppelklick oben = return
      case 5 :
      {
        ledctrl(ledgruen,1);
        ledctrl(ledgelb,1);
        ledctrl(ledrot,1);
        ledctrl(ledblau,1);
        break;
      }
      default : break;
    }

    delay(250);
    if (i== 5) { delay(500); }    // bei Doppelklick laenger leuchten
    ledctrl(ledgruen, 0);
    ledctrl(ledrot, 0);
    ledctrl(ledblau, 0);
    ledctrl(ledgelb, 0);


/*
    // Tasten auslesen ohne Wartefunktion
    if (is_buttonup())   { ledctrl(ledgruen,1); } else { ledctrl(ledgruen,0); }
    if (is_buttondwn())  { ledctrl(ledron else { ledctrl(ledrot,0); }
    if (is_buttonleft())  { ledctrl(ledblau,1); } else { ledctrl(ledblau,0); }
    if (is_buttonright())  { ledctrl(ledgelb,1); } else { ledctrl(ledgelb,0); }
*/

  }
}
